# Manifest sections
MANIFEST_APPLICATION_VERSIONS = "ApplicationVersions"
MANIFEST_CONFIGURATION_ITEMS = "ConfigurationItems"
MANIFEST_ENVIRONMENT_DEFINITIONS = "EnvironmentDefinitions"

# Application tag attributes
MANIFEST_APPLICATION_KEY = "ApplicationKey"
MANIFEST_APPLICATION_NAME = "ApplicationName"
MANIFEST_APPLICATION_VERSION_KEY = "VersionKey"
MANIFEST_APPLICATION_VERSION_NUMBER = "VersionNumber"
MANIFEST_FLAG_IS_TEST_APPLICATION = "IsTestApplication"

# Configuration item attributes
MANIFEST_MODULE_KEY = "ModuleKey"
MANIFEST_MODULE_NAME = "ModuleName"
MANIFEST_CONFIG_ITEM_KEY = "ConfigurationItemKey"
MANIFEST_CONFIG_ITEM_NAME = "ConfigurationItemName"
MANIFEST_CONFIG_ITEM_TYPE = "ConfigurationItemType"
MANIFEST_CONFIG_ITEM_VALUES = "Values"
MANIFEST_CONFIG_ITEM_TARGET_VALUE = "TargetValue"

# Environment definitions attributes
MANIFEST_ENVIRONMENT_KEY = "EnvironmentKey"
MANIFEST_ENVIRONMENT_NAME = "EnvironmentName"
MANIFEST_ENVIRONMENT_LABEL = "EnvironmentLabel"

# Manifest attributes
MANIFEST_DEPLOYMENT_NOTES = "DeploymentNotes"

# AutoGen Manifest
MANIFEST_FOLDER = "trigger_manifest"
MANIFEST_FILE = "auto_gen_trigger_manifest.json"
