# Base Architecture Dashboard API Variables
AD_HTTP_PROTO = "https"
AD_API_HOST = "aimentorstudio.outsystems.com"
AD_API_ENDPOINT = "AIMentorStudioAPI/rest"
AD_API_VERSION = "1"

# Architecture Dashboard API Endpoint Variables
AD_API_SUCCESS_CODE = 200
AD_API_BAD_REQUEST_CODE = 400
AD_API_UNAUTHORIZED_CODE = 401
AD_API_FORBIDDEN_CODE = 403
AD_API_NOT_FOUND_CODE = 404
AD_API_TOO_MANY_REQ_CODE = 429
AD_API_INTERNAL_ERROR_CODE = 500

# Application specific
AD_APP_ENDPOINT = "TechnicalDebt_Application"
AD_APP_LIMIT_DEFAULT = 50
AD_APP_SUCCESS_CODE = 200

# Level specific
AD_LEVELS_ENDPOINT = "TechnicalDebt_Level"
AD_LEVELS_SUCCESS_CODE = 200

# Categories specific
AD_CATEGORIES_ENDPOINT = "TechnicalDebt_Category"
AD_CATEGORIES_SUCCESS_CODE = 200
