# Base CICD Probe Variables
PROBE_HTTP_PROTO = "https"
PROBE_API_ENDPOINT = "CI_CDProbe/rest"
PROBE_API_VERSION = 1
PROBE_API_SSL_CERT_VERIFY = True

# Scan Endpoints
SCAN_BDD_TESTS_ENDPOINT = "ScanBDDTestEndpoints"
PROBE_SCAN_SUCCESS_CODE = 200

# Application Dependencies Endpoint
GET_APPLICATION_DEPENDENCIES_ENDPOINT = "GetApplicationDependencies"
PROBE_DEPENDENCIES_SUCCESS_CODE = 200
