# Base Properties API Variables
PROPERTIES_API_HTTP_PROTO = "https"
PROPERTIES_API_ENDPOINT = "PropertiesAPI/rest"
PROPERTIES_API_VERSION = 1
PROPERTIES_API_SSL_CERT_VERIFY = True

# Properties API Endpoints
SET_SITE_PROPERTY_VALUE_SUCCESS_CODE = 201
SET_REST_ENDPOINT_URL_SUCCESS_CODE = 201
SET_SOAP_ENDPOINT_URL_SUCCESS_CODE = 201
SET_TIMER_SCHEDULE_SUCCESS_CODE = 201

# Supported Property types
PROPERTY_TYPE_SITE_PROPERTY = "SiteProperty"
PROPERTY_TYPE_REST_ENDPOINT = "REST_Endpoint"
PROPERTY_TYPE_SOAP_ENDPOINT = "SOAP_Endpoint"
PROPERTY_TYPE_TIMER_SCHEDULE = "TimerSchedule"
